<?php

namespace SheKnows\Infuse\Core;

/**
 * Class AdTypes
 *
 * @package SheKnows\Infuse\Core
 */
class AdTypes {
	/**
	 * The types of ads supported by Boomerang that are selectable by sites.
	 * @var array
	 */
	protected $types = array(
		'banner'          => array(
			'friendlyName' => 'Banner',
			'sizes'        => array( '728x90' ),
		),
		'flexbanner'      => array(
			'friendlyName' => 'Flex Banner',
			'sizes'        => array( '728x90', '970x250', '970x90' ),
		),
		'medrec'          => array(
			'friendlyName' => 'Medium Rectangle',
			'sizes'        => array( '300x250' ),
		),
		'flexrec'         => array(
			'friendlyName' => 'Flex Rectangle',
			'sizes'        => array( '300x250', '300x600' ),
		),
		'tinybanner'      => array(
			'friendlyName' => 'Tiny Banner',
			'sizes'        => array( '300x50', '320x50' ),
		),
		'sky'             => array(
			'friendlyName' => 'Wide Skyscraper',
			'sizes'        => array( '160x600' ),
		),
		'mobileincontent' => array(
			'friendlyName' => 'Mobile In-content',
			'sizes'        => array( '300x250', '300x50', '320x50' ),
		),
		'wideincontent'   => array(
			'friendlyName' => 'Wide In-content',
			'sizes'        => array( '300x250', '728x90' ),
		),
		'widestincontent' => array(
			'friendlyName' => 'Widest In-content',
			'sizes'        => array( '300x250', '728x90', '970x90', '970x250' ),
		),
		'nativemini'      => array(
			'friendlyName' => 'Small Native In-Scroll',
		),
		'nativecontent'   => array(
			'friendlyName' => 'Native In-Content',
		),
		'nativesidebar'   => array(
			'friendlyName' => 'Native Sidebar',
		),
		'recipe'          => array(
			'friendlyName' => 'In-Recipe',
			'sizes'        => array( '300x250', '300x50', '320x50' ),
		),
	);

	/**
	 * @var Config
	 */
	protected $config;

	/**
	 * @var ErrorLogging
	 */
	protected $errorLogging;

	/**
	 * AdTypes constructor.
	 * @param Config $config
	 */
	public function __construct( Config $config, ErrorLogging $errorLogging ) {
		$this->config       = $config;
		$this->errorLogging = $errorLogging;
		$this->applyClsOptimization();
	}

	/**
	 * Applying CLS optimization to all ad types by calculating min width & height for the div inline style
	 */
	protected function applyClsOptimization() {
		try {
			$optimizeCls = $this->config->get( 'optimize_cls', 'default_optimize_cls' );
		} catch ( \Exception $exception ) {
			$this->errorLogging->record( $exception );
			$optimizeCls = false;
		}

		// Backward compatible code from existing flag = enable to be use as max size
		if ( 'enable' === $optimizeCls ) {
			$optimizeCls = 'maxsize';
		}

		if ( ! in_array( $optimizeCls, array( 'maxsize', 'minsize' ), true ) ) {
			return;
		}

		$minWidth  = 50;
		$minHeight = 50;

		foreach ( array_keys( $this->types ) as $name ) {
			$calculatedSize = $this->getAdSize( $name, $optimizeCls, $minWidth, $minHeight );

			if ( $calculatedSize[0] >= $minWidth && $calculatedSize[1] >= $minHeight ) {
				$this->types[ $name ]['style'] = sprintf(
					'min-width:%dpx;min-height:%dpx',
					$calculatedSize[0],
					$calculatedSize[1]
				);
			}
		}
	}

	/**
	 * Get the minimum or maximum size of a given slot type.
	 * NOTE: If none of the supported sizes for a given slot type meet the
	 * target minimum height/width criteria, then the returned size will be
	 * below the target.  The minWidth/minHeight targets simply ensure that
	 * the returned width/height will be the be at least the minimums if the
	 * slot supports the passed minimums.  The caller should be aware of this
	 * and handle it appropriately.
	 *
	 * @param string  $name       The name of the slot/ad type (ie "flexrec").
	 * @param string  $minMax     Set to "maxsize" to return the maximum size
	 *                            of the slot type or "minsize" for the minimum
	 *                            size.
	 * @param int     $minWidth   The target minimum width of the slot.
	 * @param int     $minHeight  The target minimum height of the slot.
	 * @return array              The calculated size [width, height].
	 */
	public function getAdSize( $name, $minMax, $minWidth = 0, $minHeight = 0 ) {
		$type = $this->getTypeByName( $name );

		$calculatedHeight = 0;
		$calculatedWidth  = 0;

		if ( $type && ! empty( $type['sizes'] ) ) {
			foreach ( $type['sizes'] as $size ) {
				list( $width, $height ) = array_map( 'intval', explode( 'x', $size ) );

				if ( 'maxsize' === $minMax ) {
					if ( $width > $calculatedWidth ) {
						$calculatedWidth = $width;
					}
					if ( $height > $calculatedHeight ) {
						$calculatedHeight = $height;
					}
				} elseif ( 'minsize' === $minMax ) {
					if ( ( 0 === $calculatedWidth ) || ( $width >= $minWidth && $width < $calculatedWidth ) || ( $calculatedWidth < $minWidth && $width < $calculatedWidth ) ) {
						$calculatedWidth = $width;
					}
					if ( ( 0 === $calculatedHeight ) || ( $height >= $minHeight && $height < $calculatedHeight ) || ( $calculatedHeight < $minHeight && $height < $calculatedHeight ) ) {
						$calculatedHeight = $height;
					}
				}
			}
		}

		return array( $calculatedWidth, $calculatedHeight );
	}

	/**
	 * Get the list of all ad types.
	 *
	 * @return array
	 */
	public function getAllTypes() {
		return $this->types;
	}

	/**
	 * Get the slot type definition for a single slot type by its name.
	 *
	 * @param  string $name The slot type to return the definition for.
	 * @return array|null
	 */
	public function getTypeByName( $name ) {
		if ( isset( $this->types[ $name ] ) ) {
			return $this->types[ $name ];
		}
	}
}
