<?php

namespace SheKnows\Infuse\Subsystems\Admiral;

use SheKnows\Infuse\Core\SubsystemLoader;

/**
 * Loads required classes for Admiral ad recovery services.
 *
 * Class Loader
 *
 * @package SheKnows\Infuse\Subsystems\Admiral
 */
class Loader extends SubsystemLoader {
	const CACHE_LIFETIME      = 21600; // 6 hours in seconds
	const FAIL_CACHE_LIFETIME = 300; // 5 minutes

	/**
	 * Initializes binding relevant classes and relevant WordPress API hooks.
	 * Note that while bound classes may not load if not required
	 */
	public function load() {
		$this->bindSubsystemClasses();
		$this->registerSubsystemHooks();
	}

	/**
	 * Makes relevant classes available at runtime. If class has not been
	 * instantiated, the class is instantiated (with dependencies added through
	 * the constructor) and handed over. If the class has been instantiated,
	 * the class is handed over.
	 */
	protected function bindSubsystemClasses() {
		$this->container->bind( __NAMESPACE__ . '\Fetch', array( $this, 'bindFetch' ) );
		$this->container->bind( __NAMESPACE__ . '\AdmiralDataCache', array( $this, 'bindAdmiralDataCache' ) );
		$this->container->bind( __NAMESPACE__ . '\Admiral', array( $this, 'bindAdmiral' ) );
	}

	/**
	 * Bind the Fetch class.
	 *
	 * @return Fetch
	 */
	public function bindFetch() {
		return new Fetch();
	}

	/**
	 * Bind the Admiral class.
	 *
	 * @return Admiral
	 */
	public function bindAdmiral() {
		$dataCache = $this->container->resolve( __NAMESPACE__ . '\AdmiralDataCache' );

		return new Admiral( $dataCache );
	}

	/**
	 * Bind the AdmiralDataCache class.
	 *
	 * @return AdmiralDataCache
	 */
	public function bindAdmiralDataCache() {
		try {
			$propertyId = $this->container->resolve( 'SheKnows\Infuse\Core\Config' )->get( 'admiral_property_id' );
		} catch ( \Exception $e ) {
			$propertyId = false;
		}

		return new AdmiralDataCache(
			$this->container->resolve( __NAMESPACE__ . '\Fetch' ),
			array(
				'propertyId' => $propertyId,
			),
			$this->container->resolve( 'SheKnows\Infuse\Core\ErrorLogging' ),
			self::CACHE_LIFETIME,
			self::FAIL_CACHE_LIFETIME,
			null,
			null
		);
	}

	/**
	 * Add hook to print Admiral on non-admin pages.
	 *
	 * @return void
	 */
	public function registerSubsystemHooks() {
		if ( is_admin() ) {
			return;
		}

		try {
			$propertyId = $this->container->resolve( 'SheKnows\Infuse\Core\Config' )->get( 'admiral_property_id' );
		} catch ( \Exception $e ) {
			return;
		}

		$enabled = $this->container
			->resolve( 'SheKnows\Infuse\Core\UpstreamServiceStatus' )
			->isEnabled( 'admiral' );

		if ( empty( $propertyId ) || ! $enabled ) {
			return;
		}

		add_action( 'wp_print_scripts', array( $this, 'printAdmiralScript' ) );
	}

	/**
	 * Print the Admiral script for this site.
	 *
	 * @return void
	 */
	public function printAdmiralScript() {
		$this->container->resolve( __NAMESPACE__ . '\Admiral' )->printScript();
	}
}
